#include <math.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <iostream.h>
#include "../matrix.hpp"
#include "../ntree.hpp"
#include "../hash.hpp"
#define MESH_IO				//include mesh text file input and output
#define MESH_BAR			//include barycentric coordinate computations
#define MESH_DEBUG
#include "../mesh.hpp"
	
int main (int argc,char** argv) {
	int c1;
	array_output=false;	
	if (argc==1) {		
		cout << "3D mesh<=>model relator for deformation. available at www.hakenberg.de" << endl;
		cout << endl;	
		cout << "usage: def3d filename1 filename2 [options]\n\n";		
		cout << "filename1 is mesh file of volumetric control mesh\n";
		cout << "filename2 is mesh file of model that is to be deformed\n";
		cout << "options: arbitrary sequence of the characters \n";
		cout << "         b - add boundary\n";
		cout << "         l - linear subdivision\n";
		cout << "         s - normal subdivision\n";
		//cout << "         o - output subdivided mesh used for computation in filename.out\n";
		cout << "to described how to yield the subdivided control mesh (not model mesh).\n";
		cout << "Operations b,l,s are performed in order of appearance.\n";
		cout << " Note: Use multiple s for smooth deformation!\n";
		cout << "Matrix D for deformation is stored in filename1.mat.\n";
		cout << "Not all model vertices have to be enclosed in which case make sure\n";
		cout << "you read the involved model vertex indices from filename1.ind.\n";		
		cout << endl;
		cout << "Example: def3d control.txt model.txt b s s s" << endl;
		cout << "hit ENTER to continue..." << endl;
		char c=getchar();
	} else {		
		mesh_init(); //call once initially, before using any mesh function
		bool flush=false;
		char fnc[80];
		mesh<float> msh(0); //default value. When adding vertices dimensionality is adjusted
		mesh<float> mod(0); 
		printf("load control mesh: %s\n",argv[1]);			
			msh.addFile(MESH_TXT,argv[1]);
			msh.content(fnc);
			printf("control mesh: %s\n",fnc);			
		mesh<float> con=msh;			
		c1=msh.arr->size[1];
		*msh.arr^=matrix<float>(c1,c1,"eye");
		printf("load model mesh: %s\n",argv[2]);
			mod.addFile(MESH_TXT,argv[2]);
			mod.content(fnc);
			printf("model mesh: %s\n",fnc);
			
		for (c1=3;c1<argc;c1++) {
			if (*argv[c1]=='b') {
				printf("add boundary\n");
				msh.addBoundary();
			}
			if (*argv[c1]=='l') {
				printf("linear subdivision\n");
				msh.subdivide(MESH_LINEAR);
			}
			if (*argv[c1]=='s') {
				printf("normal subdivision\n");
				msh.subdivide(MESH_NORMAL);
			}
			if (*argv[c1]=='o') flush=true;			
		}
		if (flush) {
			sprintf(fnc,"%s.out",argv[1]);
			printf("subdivided control mesh stored in: %s\n",fnc);
			msh.storeTxt(fnc);
			msh.content(fnc);
			printf("subdivided control mesh: %s\n",fnc);
		}
		mod.bary(con,msh);
		sprintf(fnc,"%s.mat",argv[1]);
			printf("matrix D output to:\n");
			mod.bar->storeTxt(fnc);
		sprintf(fnc,"%s.ind",argv[1]);
			printf("model vertex indices affected by multiplication are in:\n");
			mod.ocp->storeTxt(fnc);
	}
		
	return 0;
}